/*
	File moveConf.c		Hanoi Tower puzzle
	M. Kolar, 2001

Code extracting the configuration of all 3 pegs after k moves,
and which disk was moved in move k between which pegs, from the
binary representation of k,

Based on a Theorem from: T.R. Walsh, Inform. Process. Lett. 15 (1982) 64-7;
and on G.C. Rhoads algorithm.

Here disks are numbered from 1 to n, n being the largest one;
and pegs are numbered 1, 2 and 3, the start peg is 1 and
the final peg is always 3.

Input: n - number of disk
       k - move number
*/

#include <stdio.h>
#include <stdlib.h>

#define PR      (void)printf(

main(int argc, char *argv[])
/*========================*/
{
 int n, k, fr, to, *p, b, m, jm, jm1, j, i;
 
 if(argc<3) {
	fprintf(stderr,"Usage:  %s  number_of_disks  move_number\n");
	exit(1);
 }
 n = atoi(argv[1]);
 k = atoi(argv[2]);
 PR"\nHanoi Tower with %d disks\n", n);
 if(k>=(1<<n)) {
	PR"\tThere only %d moves\n", (1<<n)-1);
	exit(1);
 }

 /* T.R. Walsh:
    p[m] - peg on which the m-th disk is placed
    b - tests bits of k
    jm1, jm - values of 2 consecutive bits of k
 */
 p= (int *)malloc((n+4) * sizeof(int));
 p[n+1]= 1;  /* base of peg 1, etc. */
 p[n+2]= 2;
 p[n+3]= 3;
 b= 1<<n-1;
 p[n]= (jm= k&b) ? 3 : 1;
 for(m=n-1; m>0; m--) {
	jm1= jm>>1;
	b >>= 1;
	jm= k&b;
	if(jm == jm1) p[m]= p[m+1];
	else {
		for(i=m+2; i<n+4; i++) if(p[i] != p[m+1]) break;	
		/* disk i is on top of one of the other 2 pegs */
		if(i+m&1) p[m]= p[i]; /* i, m have opposite parity */
		else p[m]= 6-p[i]-p[m+1];
	}
 }

	if(k) {
	/* MK: j = number of the disk moved in move  k: */
	i=k; j=1; while(!(i&1)) { i>>=1; j++; }

	to= p[j]; 

	/* G.C. Rhoads: fr = peg from which disk j was moved: */
	i=k&k-1; fr=(i+i/3)&3; /* originally: fr=(x&x-1)%3; */
	/* Convert to the peg numbering used here: */
	if(fr && !(n&1)) fr= 3-fr;
	fr++;

	PR"\nMove No. %d:\n\tmove disk %d from peg %d to peg %d\n",
		k, j, fr, to);
 }

 PR"\nConfiguration after %d moves:\n", k);
 PR"\n1|"); for(m=n;m>0;m--) if(p[m]==1) PR" %d",m);
 PR"\n2|"); for(m=n;m>0;m--) if(p[m]==2) PR" %d",m);
 PR"\n3|"); for(m=n;m>0;m--) if(p[m]==3) PR" %d",m);
 PR"\n");

}
